package cn.iocoder.yudao.module.order.service.order;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.service.IService;
import cn.iocoder.yudao.framework.mybatis.core.vo.PageVO;
import cn.iocoder.yudao.module.order.dal.dataobject.orderDeparture.OrderDepartureDO;
import cn.iocoder.yudao.module.order.vo.orderDeparture.OrderDepartureCreateReqVO;
import cn.iocoder.yudao.module.order.vo.orderDeparture.OrderDepartureQueryVO;
import cn.iocoder.yudao.module.order.vo.orderDeparture.OrderDepartureUpdateReqVO;

import javax.validation.Valid;
import java.util.Collection;
import java.util.List;

/**
 * 订单始发仓库 Service 接口
 *
 * @author 系统管理员
 */
public interface OrderDepartureService extends IService<OrderDepartureDO> {

    /**
     * 创建订单始发仓库
     * @param createReqVO 创建信息
     * @return 编号
     */
    Long createOrderDeparture(@Valid OrderDepartureCreateReqVO createReqVO);

    /**
     * 更新订单始发仓库
     * @param updateReqVO 更新信息
     */
    void updateOrderDeparture(@Valid OrderDepartureUpdateReqVO updateReqVO);

    /**
     * 删除订单始发仓库
     * @param id 编号
     */
    void deleteOrderDeparture(Long id);

    /**
     * 获得订单始发仓库
     * @param id 编号
     * @return 订单始发仓库
     */
    OrderDepartureDO getOrderDeparture(Long id);

    /**
     * 获得订单始发仓库列表
     * @param ids 编号
     * @return 订单始发仓库列表
     */
    List<OrderDepartureDO> getOrderDepartureList(Collection<Long> ids);

    /**
     * 获得订单始发仓库分页
     * @param page 分页查询
     * @param query 查询
     * @return 订单始发仓库分页
     */
    PageResult<OrderDepartureDO> getOrderDeparturePage(OrderDepartureQueryVO query, PageVO page);

    /**
     * 获得订单始发仓库列表, 用于 Excel 导出
     * @param query 查询
     * @return 订单始发仓库列表
     */
    List<OrderDepartureDO> getOrderDepartureList(OrderDepartureQueryVO query);

    OrderDepartureDO getByOrderId(Long orderId);

    List<OrderDepartureDO> getByOrderIdList(List<Long> orderIdList);
}
